// (C) 2024 GoodData Corporation
import React, { useMemo, useEffect } from "react";
import { injectIntl, useIntl } from "react-intl";
import noop from "lodash/noop.js";
import { LoadingComponent as SDKLoadingComponent, ErrorComponent as SDKErrorComponent, useCancelablePromise, newErrorMapping, IntlWrapper, convertError, ErrorCodes, BucketNames, DataViewFacade, } from "@gooddata/sdk-ui";
import { bucketsFind, isAttribute } from "@gooddata/sdk-model";
import { ThemeContextProvider, useTheme, withTheme } from "@gooddata/sdk-ui-theme-provider";
import { RepeaterChart } from "./internal/RepeaterChart.js";
import { getValidColorPalette, ColorFactory } from "../../highcharts/index.js";
import { getWindowSize } from "./internal/repeaterAgGridDataSource.js";
export * from "./publicTypes.js";
export * from "./columnWidths.js";
export const CoreRepeaterImpl = (props) => {
    const { execution, ErrorComponent = SDKErrorComponent, LoadingComponent = SDKLoadingComponent, onLoadingChanged, pushData, onError, onColumnResized, config = {}, drillableItems = [], onDrill = noop, } = props;
    const intl = useIntl();
    const { result, error } = useCancelablePromise({
        promise: async () => {
            const { offset, size } = getWindowSize(execution.definition.dimensions.length);
            const executionResult = await execution.execute();
            const dataWindow = await executionResult.readWindow(offset, size);
            return DataViewFacade.for(dataWindow);
        },
        onLoading: () => {
            onLoadingChanged?.({ isLoading: true });
        },
        onSuccess: (dataView) => {
            onLoadingChanged?.({ isLoading: false });
            pushData?.({ dataView: dataView.dataView });
        },
        onError: (error) => {
            onLoadingChanged?.({ isLoading: false });
            onError?.(convertError(error));
        },
    }, [execution.fingerprint()]);
    const configWithColorPalette = useMemo(() => {
        const colorPalette = getValidColorPalette(config);
        return {
            ...config,
            colorPalette,
        };
    }, [config]);
    const theme = useTheme();
    const colorMapping = JSON.stringify([
        configWithColorPalette.colorPalette,
        configWithColorPalette.colorMapping,
    ]);
    useEffect(() => {
        if (result) {
            const colorStrategy = ColorFactory.getColorStrategy(configWithColorPalette.colorPalette, configWithColorPalette.colorMapping, null, null, null, result, "repeater", theme);
            const colorAssignment = colorStrategy.getColorAssignment();
            pushData?.({
                colors: {
                    colorAssignments: colorAssignment,
                    colorPalette: configWithColorPalette.colorPalette,
                },
            });
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [theme, colorMapping, pushData, result]);
    useEffect(() => {
        if (result) {
            const columns = bucketsFind(result.definition.buckets, BucketNames.COLUMNS);
            pushData?.({
                availableDrillTargets: {
                    attributes: result
                        .meta()
                        .attributeDescriptors()
                        .filter((descriptor) => columns.items.find((item) => {
                        if (isAttribute(item)) {
                            return (item.attribute.localIdentifier ===
                                descriptor.attributeHeader.localIdentifier);
                        }
                        return false;
                    }))
                        .map((descriptor) => {
                        return {
                            attribute: descriptor,
                            intersectionAttributes: [descriptor],
                        };
                    }),
                    measures: [],
                },
            });
        }
    }, [pushData, result]);
    if (error) {
        const convertedError = convertError(error);
        const errorMessage = convertedError.getMessage();
        const errorMap = newErrorMapping(intl);
        const errorProps = errorMap[Object.prototype.hasOwnProperty.call(errorMap, error) ? error : ErrorCodes.UNKNOWN_ERROR];
        return React.createElement(ErrorComponent, { code: errorMessage, ...errorProps });
    }
    if (!result) {
        return React.createElement(LoadingComponent, null);
    }
    const columns = bucketsFind(result.definition.buckets, BucketNames.COLUMNS);
    if (!columns || columns.items.length === 0) {
        const err = new Error("Repeater chart requires at least one column");
        const convertedError = convertError(err);
        const errorMessage = convertedError.getMessage();
        const errorMap = newErrorMapping(intl);
        const errorProps = errorMap[Object.prototype.hasOwnProperty.call(errorMap, err) ? err : ErrorCodes.UNKNOWN_ERROR];
        return React.createElement(ErrorComponent, { code: errorMessage, ...errorProps });
    }
    return (React.createElement(RepeaterChart, { dataView: result, drillableItems: drillableItems, onDrill: onDrill, config: configWithColorPalette, onError: onError, onColumnResized: onColumnResized }));
};
const CoreRepeaterWithIntl = injectIntl(withTheme(CoreRepeaterImpl));
/**
 * @internal
 */
export const CoreRepeater = (props) => (React.createElement(ThemeContextProvider, { theme: props.theme, themeIsLoading: false },
    React.createElement(IntlWrapper, { locale: props.locale },
        React.createElement(CoreRepeaterWithIntl, { ...props }))));
//# sourceMappingURL=CoreRepeater.js.map