// (C) 2024 GoodData Corporation
import React, { useMemo, useState } from "react";
import { AgGridReact } from "@ag-grid-community/react";
import { AllCommunityModules } from "@ag-grid-community/all-modules";
import { BucketNames, LoadingComponent, emptyHeaderTitleFromIntl } from "@gooddata/sdk-ui";
import { AgGridDatasource } from "./repeaterAgGridDataSource.js";
import { attributeLocalId, bucketsFind, isMeasure, measureLocalId, } from "@gooddata/sdk-model";
import { useTheme } from "@gooddata/sdk-ui-theme-provider";
import { useIntl } from "react-intl";
import { Icon } from "@gooddata/sdk-ui-kit";
import cx from "classnames";
import stringify from "json-stable-stringify";
import { useResizing, MANUALLY_SIZED_MIN_WIDTH } from "../hooks/useResizing.js";
import { InlineLineChart } from "./InlineLineChart.js";
import { InlineColumnChart } from "./InlineColumnChart.js";
import isNil from "lodash/isNil.js";
import { useDrilling } from "../hooks/useDrilling.js";
const DEFAULT_COL_DEF = { resizable: true };
export const RepeaterChart = (props) => {
    const { dataView, onError, config } = props;
    const dataSource = useMemo(() => new AgGridDatasource(dataView, { onError }, config), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [dataView.fingerprint(), onError, config]);
    const rowHeight = getRowHeight(config);
    const items = useMemo(() => {
        const columnsBucket = bucketsFind(dataView.definition.buckets, BucketNames.COLUMNS);
        return columnsBucket.items ?? [];
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [dataView.fingerprint()]);
    const columnDefs = useMemo(() => {
        let measureIndex = 0;
        return items.map((bucketItem) => {
            const sharedColDef = {
                headerName: getRepeaterColumnTitle(bucketItem, dataView, config.enableAliasAttributeLabel),
                field: getRepeaterColumnId(bucketItem),
                cellClass: "gd-cell",
                minWidth: MANUALLY_SIZED_MIN_WIDTH,
            };
            if (isMeasure(bucketItem)) {
                const localId = measureLocalId(bucketItem);
                const measureTitle = getMetricTitle(bucketItem, dataView);
                const viewByAttributeLocalId = getViewByAttributeLocalId(dataView);
                const viewByAttributeTitle = getViewByAttributeTitle(dataView);
                const measureColor = getMetricColor(bucketItem, dataView, config, measureIndex);
                measureIndex = measureIndex + 1;
                return {
                    ...sharedColDef,
                    cellRenderer: function CellRenderer(props) {
                        const { node } = props;
                        const measureColumnData = props.value;
                        const isLoading = node.id === undefined;
                        return (React.createElement(MeasureCellRenderer, { measureColumnData: measureColumnData, isLoading: isLoading }));
                    },
                    valueGetter: (params) => {
                        return {
                            measureLocalId: localId,
                            measureTitle,
                            measureDataPoints: params.data?.[localId],
                            viewByAttributeTitle,
                            viewByAttributeHeaderItems: params.data?.[viewByAttributeLocalId] ?? [],
                            config,
                            measureColor,
                        };
                    },
                };
            }
            else {
                const attributeDescriptors = dataView.meta().attributeDescriptors();
                const localIdentifier = attributeLocalId(bucketItem);
                const attributeDescriptor = attributeDescriptors.find((descriptor) => descriptor.attributeHeader.localIdentifier === localIdentifier);
                const renderingType = getAttributeRenderingType(attributeDescriptor);
                return {
                    ...sharedColDef,
                    cellRenderer: function CellRenderer(props) {
                        const { node } = props;
                        const attributeColumnData = props.value;
                        const isLoading = node.id === undefined;
                        return (React.createElement(AttributeCellRenderer, { attributeColumnData: attributeColumnData, isLoading: isLoading }));
                    },
                    valueGetter: (params) => {
                        return {
                            attributeLocalId: localIdentifier,
                            value: params.data?.[localIdentifier]?.name,
                            config,
                            renderingType,
                        };
                    },
                };
            }
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [
        items,
        config?.cellVerticalAlign,
        config?.cellTextWrapping,
        config?.cellImageSizing,
        config?.hyperLinks,
        config?.inlineVisualizations,
    ]);
    const { onColumnResized, onGridReady: onResizingGridReady, containerRef, } = useResizing(columnDefs, items, props);
    const { onCellClicked, onGridReady: onDrillingGridReady } = useDrilling(columnDefs, items, props);
    return (React.createElement("div", { className: "gd-repeater ag-theme-balham s-repeater", ref: containerRef },
        React.createElement(AgGridReact, { key: stringify({
                rowHeight,
                cellVerticalAlign: config?.cellVerticalAlign,
                cellTextWrapping: config?.cellTextWrapping,
                cellImageSizing: config?.cellImageSizing,
                dataView: dataView.fingerprint(),
                hyperLinks: config?.hyperLinks,
                onError: onError?.toString(),
            }), defaultColDef: DEFAULT_COL_DEF, modules: AllCommunityModules, columnDefs: columnDefs, rowClass: "gd-table-row", datasource: dataSource, rowModelType: "infinite", rowHeight: rowHeight, suppressCellFocus: true, suppressMovableColumns: true, onCellClicked: onCellClicked, onGridReady: (e) => {
                onResizingGridReady(e);
                onDrillingGridReady(e);
            }, onColumnResized: onColumnResized })));
};
function RepeaterImage({ src }) {
    const [imageLoadError, setImageLoadError] = useState(false);
    if (!src || imageLoadError) {
        return (React.createElement("div", { className: "gd-repeater-image-empty" },
            React.createElement(Icon.Image, null)));
    }
    return React.createElement("img", { className: "gd-repeater-image", src: src, alt: src, onError: () => setImageLoadError(true) });
}
function RepeaterHyperLink({ value, hyperlinkStaticText }) {
    const intl = useIntl();
    return value ? (React.createElement("a", { className: "gd-repeater-link", href: value, target: "_blank", rel: "noopener noreferrer" }, hyperlinkStaticText ? hyperlinkStaticText : value)) : (emptyHeaderTitleFromIntl(intl));
}
function MeasureCellRenderer({ measureColumnData, isLoading, }) {
    const { measureLocalId, measureTitle, config, viewByAttributeTitle, measureDataPoints, viewByAttributeHeaderItems, } = measureColumnData;
    const rowHeight = getRowHeight(config);
    const visualizationType = getInlineVisualizationType(measureLocalId, config);
    const theme = useTheme();
    const color = theme?.table?.loadingIconColor ?? theme?.palette?.complementary?.c6 ?? undefined;
    if (isLoading) {
        return React.createElement(LoadingComponent, { color: color, width: 36, imageHeight: 8, height: rowHeight, speed: 2 });
    }
    if (visualizationType === "line") {
        return (React.createElement(InlineLineChart, { metricTitle: measureTitle, sliceTitle: viewByAttributeTitle, data: measureDataPoints, color: measureColumnData.measureColor, headerItems: viewByAttributeHeaderItems, height: rowHeight }));
    }
    else if (visualizationType === "column") {
        return (React.createElement(InlineColumnChart, { metricTitle: measureTitle, sliceTitle: viewByAttributeTitle, data: measureDataPoints, color: measureColumnData.measureColor, headerItems: viewByAttributeHeaderItems, height: rowHeight }));
    }
    const verticalAlign = getVerticalAlign(config);
    const textWrapping = getTextWrapping(config);
    return (React.createElement("div", { className: cx("gd-repeater-cell-wrapper", `gd-vertical-align-${verticalAlign}`, `gd-text-wrapping-${textWrapping}`) }, measureDataPoints.find((point) => !isNil(point.value))?.formattedValue));
}
function AttributeCellRenderer({ attributeColumnData, isLoading, }) {
    const { config, attributeLocalId, value, renderingType } = attributeColumnData;
    const rowHeight = getRowHeight(config);
    const intl = useIntl();
    const theme = useTheme();
    const color = theme?.table?.loadingIconColor ?? theme?.palette?.complementary?.c6 ?? undefined;
    if (isLoading) {
        return React.createElement(LoadingComponent, { color: color, width: 36, imageHeight: 8, height: rowHeight, speed: 2 });
    }
    let renderValue = value || emptyHeaderTitleFromIntl(intl);
    if (renderingType === "text") {
        renderValue = React.createElement("span", null, renderValue);
    }
    else if (renderingType === "hyperlink") {
        const hyperlinkStaticText = getHyperLinkStaticElementsText(attributeLocalId, config);
        renderValue = React.createElement(RepeaterHyperLink, { hyperlinkStaticText: hyperlinkStaticText, value: value });
    }
    else if (renderingType === "image") {
        renderValue = React.createElement(RepeaterImage, { src: value });
    }
    const verticalAlign = getVerticalAlign(config);
    const textWrapping = getTextWrapping(config);
    const imageSizing = getImageSizing(config);
    return (React.createElement("div", { className: cx("gd-repeater-cell-wrapper", `gd-vertical-align-${verticalAlign}`, `gd-row-height-${rowHeight}`, `gd-text-wrapping-${textWrapping}`, `gd-image-sizing-${imageSizing}`) }, renderValue));
}
const SMALL_ROW_HEIGHT = 25;
const MEDIUM_ROW_HEIGHT = 55;
const LARGE_ROW_HEIGHT = 85;
function getRowHeight(config) {
    const rowHeight = config?.rowHeight;
    switch (rowHeight) {
        case "small":
            return SMALL_ROW_HEIGHT;
        case "medium":
            return MEDIUM_ROW_HEIGHT;
        case "large":
            return LARGE_ROW_HEIGHT;
        default:
            return SMALL_ROW_HEIGHT;
    }
}
function getVerticalAlign(config) {
    return config?.cellVerticalAlign ?? "top";
}
function getTextWrapping(config) {
    return config?.cellTextWrapping ?? "clip";
}
function getImageSizing(config) {
    return config?.cellImageSizing ?? "fit";
}
function getRepeaterColumnTitle(columnBucketItem, dataView, enableAliasAttributeLabel) {
    if (isMeasure(columnBucketItem)) {
        return getMetricTitle(columnBucketItem, dataView);
    }
    return getAttributeTitle(columnBucketItem, dataView, enableAliasAttributeLabel);
}
function getRepeaterColumnId(columnBucketItem) {
    if (isMeasure(columnBucketItem)) {
        return measureLocalId(columnBucketItem);
    }
    return attributeLocalId(columnBucketItem);
}
function getMetricColor(measure, dataView, config, index) {
    const descriptor = getMetricDescriptor(measure, dataView);
    const colorPalette = config?.colorPalette;
    const colorMappingForMeasure = config?.colorMapping?.find((cm) => cm.predicate?.(descriptor, { dv: dataView }));
    let color;
    if (colorMappingForMeasure?.color?.type === "rgb") {
        const rgbColor = colorMappingForMeasure.color.value;
        color = `rgb(${rgbColor.r}, ${rgbColor.g}, ${rgbColor.b})`;
    }
    else if (colorMappingForMeasure?.color?.type === "guid") {
        const paletteColor = colorPalette?.find((c) => c.guid === colorMappingForMeasure.color.value);
        if (paletteColor) {
            const rgbColor = paletteColor.fill;
            color = `rgb(${rgbColor.r}, ${rgbColor.g}, ${rgbColor.b})`;
        }
    }
    if (!color) {
        const paletteColor = colorPalette?.[index];
        if (paletteColor) {
            const rgbColor = paletteColor.fill;
            color = `rgb(${rgbColor.r}, ${rgbColor.g}, ${rgbColor.b})`;
        }
        else {
            color = "#14B2E2";
        }
    }
    return color;
}
function getMetricTitle(measure, dataView) {
    const measureDescriptor = getMetricDescriptor(measure, dataView);
    return measureDescriptor?.measureHeaderItem?.name;
}
function getMetricDescriptor(measure, dataView) {
    const localId = measureLocalId(measure);
    const measureDescriptors = dataView.meta().measureDescriptors();
    return measureDescriptors.find((descriptor) => descriptor.measureHeaderItem.localIdentifier === localId);
}
function getAttributeTitle(attribute, dataView, enableAliasAttributeLabel = false) {
    const localId = attributeLocalId(attribute);
    const attributeDescriptors = dataView.meta().attributeDescriptors();
    const attributeDescriptor = attributeDescriptors.find((descriptor) => descriptor.attributeHeader.localIdentifier === localId);
    return enableAliasAttributeLabel
        ? attributeDescriptor?.attributeHeader?.formOf.name
        : attributeDescriptor?.attributeHeader?.name;
}
function getInlineVisualizationType(measureLocalId, config) {
    return config?.inlineVisualizations?.[measureLocalId]?.type ?? "metric";
}
function getViewByAttributeLocalId(dataView) {
    const viewBucket = bucketsFind(dataView.definition.buckets, BucketNames.VIEW);
    const viewAttribute = viewBucket?.items?.[0];
    return viewAttribute ? attributeLocalId(viewAttribute) : undefined;
}
function getViewByAttributeDescriptor(dataView) {
    const attributeDescriptors = dataView.meta().attributeDescriptors();
    const viewAttributeLocalId = getViewByAttributeLocalId(dataView);
    return attributeDescriptors.find((d) => d.attributeHeader.localIdentifier === viewAttributeLocalId);
}
function getViewByAttributeTitle(dataView) {
    return getViewByAttributeDescriptor(dataView)?.attributeHeader?.name;
}
function getAttributeRenderingType(attributeDescriptor) {
    const labelType = attributeDescriptor.attributeHeader.labelType;
    if (labelType === "GDC.image") {
        return "image";
    }
    else if (labelType === "GDC.link") {
        return "hyperlink";
    }
    return "text";
}
function getHyperLinkStaticElementsText(attributeLocalId, config) {
    return config?.hyperLinks?.[attributeLocalId]?.staticElementsText;
}
//# sourceMappingURL=RepeaterChart.js.map